@@ -8,6 +8,7 @@
 #include <algorithm>
 #include <atomic>
 
+#include "base/allocator/partition_allocator/partition_alloc_check.h"
 #include "base/allocator/partition_allocator/partition_alloc_config.h"
 #include "base/allocator/partition_allocator/yield_processor.h"
 #include "base/base_export.h"
@@ -16,13 +17,52 @@
 #include "build/build_config.h"
 
 #if defined(OS_WIN)
-#include <windows.h>
+#include "base/win/windows_types.h"
+#endif
+
+#if defined(OS_POSIX)
+#include <errno.h>
+#include <pthread.h>
+#endif
+
+#if defined(OS_APPLE)
+#include <os/lock.h>
+
+// os_unfair_lock is available starting with OS X 10.12, and Chromium targets
+// 10.11 at the minimum, so the symbols are not always available *at runtime*.
+// But we build with a 11.x SDK, so it's always in the headers.
+//
+// However, since the majority of clients have at least 10.12 (released late
+// 2016), we declare the symbols here, marking them weak. They will be nullptr
+// on 10.11, and defined on more recent versions.
+
+// Silence the compiler warning, here and below.
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunguarded-availability"
+
+#define PA_WEAK __attribute__((weak))
+
+extern "C" {
+
+PA_WEAK void os_unfair_lock_lock(os_unfair_lock_t lock);
+PA_WEAK bool os_unfair_lock_trylock(os_unfair_lock_t lock);
+PA_WEAK void os_unfair_lock_unlock(os_unfair_lock_t lock);
+}
+
+#pragma clang diagnostic pop
+
+#endif  // defined(OS_APPLE)
+
+#if defined(OS_FUCHSIA)
+#include <lib/sync/mutex.h>
 #endif
 
-#if defined(PA_HAS_SPINNING_MUTEX)
 namespace base {
 namespace internal {
 
+// The behavior of this class depends on whether PA_HAS_FAST_MUTEX is defined.
+// 1. When it is defined:
+//
 // Simple spinning lock. It will spin in user space a set number of times before
 // going into the kernel to sleep.
 //
@@ -44,6 +84,11 @@ namespace internal {
 // As an interesting side-effect to be used in the allocator, this code does not
 // make any allocations, locks are small with a constexpr constructor and no
 // destructor.
+//
+// 2. Otherwise: This is a simple SpinLock, in the sense that it does not have
+// any awareness of other threads' behavior. One exception: x86 macOS uses
+// os_unfair_lock() if available, which is the case for macOS >= 10.12, that is
+// most clients.
 class LOCKABLE BASE_EXPORT SpinningMutex {
  public:
   inline constexpr SpinningMutex();
@@ -53,12 +98,14 @@ class LOCKABLE BASE_EXPORT SpinningMutex {
   void AssertAcquired() const {}  // Not supported.
 
  private:
-  void LockSlow();
+  void LockSlow() EXCLUSIVE_LOCK_FUNCTION();
 
   // Same as SpinLock, not scientifically calibrated. Consider lowering later,
   // as the slow path has better characteristics than SpinLocks's.
   static constexpr int kSpinCount = 1000;
 
+#if defined(PA_HAS_FAST_MUTEX)
+
 #if defined(PA_HAS_LINUX_KERNEL)
   void FutexWait();
   void FutexWake();
@@ -68,8 +115,31 @@ class LOCKABLE BASE_EXPORT SpinningMutex {
   static constexpr int kLockedContended = 2;
 
   std::atomic<int32_t> state_{kUnlocked};
-#else
-  SRWLOCK lock_ = SRWLOCK_INIT;
+#elif defined(OS_WIN)
+  CHROME_SRWLOCK lock_ = SRWLOCK_INIT;
+#elif defined(OS_POSIX)
+  pthread_mutex_t lock_ = PTHREAD_MUTEX_INITIALIZER;
+#elif defined(OS_FUCHSIA)
+  sync_mutex lock_;
+#endif
+
+#else  // defined(PA_HAS_FAST_MUTEX)
+  std::atomic<bool> lock_{false};
+
+#if defined(OS_APPLE)
+
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunguarded-availability"
+  os_unfair_lock unfair_lock_ = OS_UNFAIR_LOCK_INIT;
+#pragma clang diagnostic pop
+
+#endif
+
+  // Spinlock-like, fallback.
+  ALWAYS_INLINE bool TrySpinLock();
+  ALWAYS_INLINE void ReleaseSpinLock();
+  void LockSlowSpinLock();
+
 #endif
 };
 
@@ -104,6 +174,8 @@ ALWAYS_INLINE void SpinningMutex::Acquire() {
 
 inline constexpr SpinningMutex::SpinningMutex() = default;
 
+#if defined(PA_HAS_FAST_MUTEX)
+
 #if defined(PA_HAS_LINUX_KERNEL)
 
 ALWAYS_INLINE bool SpinningMutex::Try() {
@@ -141,7 +213,7 @@ ALWAYS_INLINE void SpinningMutex::Release() {
   }
 }
 
-#else
+#elif defined(OS_WIN)
 
 ALWAYS_INLINE bool SpinningMutex::Try() {
   return !!::TryAcquireSRWLockExclusive(reinterpret_cast<PSRWLOCK>(&lock_));
@@ -151,10 +223,90 @@ ALWAYS_INLINE void SpinningMutex::Release() {
   ::ReleaseSRWLockExclusive(reinterpret_cast<PSRWLOCK>(&lock_));
 }
 
+#elif defined(OS_POSIX)
+
+ALWAYS_INLINE bool SpinningMutex::Try() {
+  int retval = pthread_mutex_trylock(&lock_);
+  PA_DCHECK(retval == 0 || retval == EBUSY);
+  return retval == 0;
+}
+
+ALWAYS_INLINE void SpinningMutex::Release() {
+  int retval = pthread_mutex_unlock(&lock_);
+  PA_DCHECK(retval == 0);
+}
+
+#elif defined(OS_FUCHSIA)
+
+ALWAYS_INLINE bool SpinningMutex::Try() {
+  return sync_mutex_trylock(&lock_) == ZX_OK;
+}
+
+ALWAYS_INLINE void SpinningMutex::Release() {
+  sync_mutex_unlock(&lock_);
+}
+
 #endif
 
+#else  // defined(PA_HAS_FAST_MUTEX)
+
+ALWAYS_INLINE bool SpinningMutex::TrySpinLock() {
+  // Possibly faster than CAS. The theory is that if the cacheline is shared,
+  // then it can stay shared, for the contended case.
+  return !lock_.load(std::memory_order_relaxed) &&
+         !lock_.exchange(true, std::memory_order_acquire);
+}
+
+ALWAYS_INLINE void SpinningMutex::ReleaseSpinLock() {
+  lock_.store(false, std::memory_order_release);
+}
+
+#if defined(OS_APPLE)
+
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunguarded-availability"
+
+ALWAYS_INLINE bool SpinningMutex::Try() {
+  if (LIKELY(os_unfair_lock_trylock))
+    return os_unfair_lock_trylock(&unfair_lock_);
+
+  return TrySpinLock();
+}
+
+ALWAYS_INLINE void SpinningMutex::Release() {
+  // Always testing trylock(), since the definitions are all or nothing.
+  if (LIKELY(os_unfair_lock_trylock))
+    return os_unfair_lock_unlock(&unfair_lock_);
+
+  return ReleaseSpinLock();
+}
+
+ALWAYS_INLINE void SpinningMutex::LockSlow() {
+  if (LIKELY(os_unfair_lock_trylock))
+    return os_unfair_lock_lock(&unfair_lock_);
+  return LockSlowSpinLock();
+}
+
+#pragma clang diagnostic pop
+
+#else
+ALWAYS_INLINE bool SpinningMutex::Try() {
+  return TrySpinLock();
+}
+
+ALWAYS_INLINE void SpinningMutex::Release() {
+  return ReleaseSpinLock();
+}
+
+ALWAYS_INLINE void SpinningMutex::LockSlow() {
+  return LockSlowSpinLock();
+}
+
+#endif  // defined(OS_APPLE)
+
+#endif  // defined(PA_HAS_FAST_MUTEX)
+
 }  // namespace internal
 }  // namespace base
-#endif  // defined(PA_HAS_SPINNING_MUTEX)
 
 #endif  // BASE_ALLOCATOR_PARTITION_ALLOCATOR_SPINNING_MUTEX_H_
