@@ -109,7 +109,7 @@ void IncreaseFdLimitTo(unsigned int max_descriptors) {
 namespace {
 
 size_t GetMallocUsageMallinfo() {
-#if defined(OS_LINUX) && defined(__UCLIBC__)
+#if defined(__MUSL__)
   return 0;
 #else
 #if defined(__GLIBC__) && defined(__GLIBC_PREREQ)
