@@ -319,15 +319,17 @@ MockDnsClientRule::Result::Result(ResultType type,
     : type(type), response(std::move(response)), net_error(net_error) {}
 
 MockDnsClientRule::Result::Result(DnsResponse response)
-    : type(OK), response(std::move(response)), net_error(absl::nullopt) {}
+    : type(ResultType::kOk),
+      response(std::move(response)),
+      net_error(absl::nullopt) {}
 
-MockDnsClientRule::Result::Result(Result&& result) = default;
+MockDnsClientRule::Result::Result(Result&&) = default;
+
+MockDnsClientRule::Result& MockDnsClientRule::Result::operator=(Result&&) =
+    default;
 
 MockDnsClientRule::Result::~Result() = default;
 
-MockDnsClientRule::Result& MockDnsClientRule::Result::operator=(
-    Result&& result) = default;
-
 MockDnsClientRule::MockDnsClientRule(const std::string& prefix,
                                      uint16_t qtype,
                                      bool secure,
@@ -357,7 +359,7 @@ class MockDnsTransactionFactory::MockTransaction
                   ResolveContext* resolve_context,
                   bool fast_timeout,
                   DnsTransactionFactory::CallbackType callback)
-      : result_(MockDnsClientRule::FAIL),
+      : result_(MockDnsClientRule::ResultType::kFail),
         hostname_(std::move(hostname)),
         qtype_(qtype),
         callback_(std::move(callback)),
@@ -390,8 +392,8 @@ class MockDnsTransactionFactory::MockTransaction
           absl::optional<DnsQuery> query(absl::in_place, 22 /* id */, dns_name,
                                          qtype_);
           switch (result->type) {
-            case MockDnsClientRule::NODOMAIN:
-            case MockDnsClientRule::EMPTY:
+            case MockDnsClientRule::ResultType::kNoDomain:
+            case MockDnsClientRule::ResultType::kEmpty:
               DCHECK(!result->response);  // Not expected to be provided.
               authority_records = {BuildTestDnsRecord(
                   hostname_, dns_protocol::kTypeSOA, "fake rdata")};
@@ -401,29 +403,29 @@ class MockDnsTransactionFactory::MockTransaction
                   authority_records,
                   std::vector<DnsResourceRecord>() /* additional_records */,
                   query,
-                  result->type == MockDnsClientRule::NODOMAIN
+                  result->type == MockDnsClientRule::ResultType::kNoDomain
                       ? dns_protocol::kRcodeNXDOMAIN
                       : 0);
               break;
-            case MockDnsClientRule::FAIL:
+            case MockDnsClientRule::ResultType::kFail:
               if (result->response)
                 SetResponse(result);
               break;
-            case MockDnsClientRule::TIMEOUT:
+            case MockDnsClientRule::ResultType::kTimeout:
               DCHECK(!result->response);  // Not expected to be provided.
               break;
-            case MockDnsClientRule::SLOW:
+            case MockDnsClientRule::ResultType::kSlow:
               if (!fast_timeout)
                 SetResponse(result);
               break;
-            case MockDnsClientRule::OK:
+            case MockDnsClientRule::ResultType::kOk:
               SetResponse(result);
               break;
-            case MockDnsClientRule::MALFORMED:
+            case MockDnsClientRule::ResultType::kMalformed:
               DCHECK(!result->response);  // Not expected to be provided.
               result_.response = CreateMalformedResponse(hostname_, qtype_);
               break;
-            case MockDnsClientRule::UNEXPECTED:
+            case MockDnsClientRule::ResultType::kUnexpected:
               ADD_FAILURE()
                   << "Unexpected DNS transaction created for hostname "
                   << hostname_;
@@ -484,8 +486,8 @@ class MockDnsTransactionFactory::MockTransaction
 
   void Finish() {
     switch (result_.type) {
-      case MockDnsClientRule::NODOMAIN:
-      case MockDnsClientRule::FAIL: {
+      case MockDnsClientRule::ResultType::kNoDomain:
+      case MockDnsClientRule::ResultType::kFail: {
         int error = result_.net_error.value_or(ERR_NAME_NOT_RESOLVED);
         DCHECK_NE(error, OK);
         std::move(callback_).Run(this, error,
@@ -493,19 +495,19 @@ class MockDnsTransactionFactory::MockTransaction
                                  absl::nullopt);
         break;
       }
-      case MockDnsClientRule::EMPTY:
-      case MockDnsClientRule::OK:
-      case MockDnsClientRule::MALFORMED:
+      case MockDnsClientRule::ResultType::kEmpty:
+      case MockDnsClientRule::ResultType::kOk:
+      case MockDnsClientRule::ResultType::kMalformed:
         DCHECK(!result_.net_error.has_value());
         std::move(callback_).Run(
             this, OK, base::OptionalOrNullptr(result_.response), absl::nullopt);
         break;
-      case MockDnsClientRule::TIMEOUT:
+      case MockDnsClientRule::ResultType::kTimeout:
         DCHECK(!result_.net_error.has_value());
         std::move(callback_).Run(this, ERR_DNS_TIMED_OUT, nullptr,
                                  absl::nullopt);
         break;
-      case MockDnsClientRule::SLOW:
+      case MockDnsClientRule::ResultType::kSlow:
         if (result_.response) {
           std::move(callback_).Run(
               this, result_.net_error.value_or(OK),
@@ -517,7 +519,7 @@ class MockDnsTransactionFactory::MockTransaction
                                    absl::nullopt);
         }
         break;
-      case MockDnsClientRule::UNEXPECTED:
+      case MockDnsClientRule::ResultType::kUnexpected:
         ADD_FAILURE() << "Unexpected DNS transaction completed for hostname "
                       << hostname_;
         break;
