@@ -68,13 +68,6 @@ class NET_EXPORT NetworkQualityEstimatorParams {
     return weight_multiplier_per_second_;
   }
 
-  // Returns the factor by which the weight of an observation reduces for every
-  // signal strength level difference between the current signal strength, and
-  // the signal strength at the time when the observation was taken.
-  double weight_multiplier_per_signal_strength_level() const {
-    return weight_multiplier_per_signal_strength_level_;
-  }
-
   // Returns an unset value if the effective connection type has not been forced
   // via the |params| provided to this class. Otherwise, returns a value set to
   // the effective connection type that has been forced. Forced ECT can be
@@ -240,12 +233,6 @@ class NET_EXPORT NetworkQualityEstimatorParams {
   // quality estimate.
   bool use_end_to_end_rtt() const { return use_end_to_end_rtt_; }
 
-  // Return true if ECT value should be capped based on the current signal
-  // strength.
-  bool cap_ect_based_on_signal_strength() const {
-    return cap_ect_based_on_signal_strength_;
-  }
-
   // Returns a multiplier which is used to clamp Kbps on slow connections. For
   // a given ECT, the upper bound on Kbps is computed based on this returned
   // multiplier and the typical Kbps for the given ECT. If
@@ -255,18 +242,6 @@ class NET_EXPORT NetworkQualityEstimatorParams {
     return upper_bound_typical_kbps_multiplier_;
   }
 
-  // Returns true if the signal strength or detailed network ID should be
-  // queried.
-  bool get_signal_strength_and_detailed_network_id() const {
-    return get_signal_strength_and_detailed_network_id_;
-  }
-
-  // Returns the minimum duration between two consecutuve calls for querying the
-  // current WiFi network's signal strength.
-  base::TimeDelta wifi_signal_strength_query_interval() const {
-    return wifi_signal_strength_query_interval_;
-  }
-
   // Returns true if RTTs should be adjusted based on RTT counts.
   // If there are not enough transport RTT samples, end-to-end RTT samples and
   // the cached estimates are unavailble/too stale, then the computed value of
@@ -289,7 +264,6 @@ class NET_EXPORT NetworkQualityEstimatorParams {
   const int throughput_min_transfer_size_kilobytes_;
   const double throughput_hanging_requests_cwnd_size_multiplier_;
   const double weight_multiplier_per_second_;
-  const double weight_multiplier_per_signal_strength_level_;
   absl::optional<EffectiveConnectionType> forced_effective_connection_type_;
   const bool forced_effective_connection_type_on_cellular_only_;
   bool persistent_cache_reading_enabled_;
@@ -308,10 +282,7 @@ class NET_EXPORT NetworkQualityEstimatorParams {
   const bool add_default_platform_observations_;
   const base::TimeDelta socket_watchers_min_notification_interval_;
   const bool use_end_to_end_rtt_;
-  const bool cap_ect_based_on_signal_strength_;
   const double upper_bound_typical_kbps_multiplier_;
-  const bool get_signal_strength_and_detailed_network_id_;
-  const base::TimeDelta wifi_signal_strength_query_interval_;
   const bool adjust_rtt_based_on_rtt_counts_;
 
   bool use_small_responses_;
