@@ -211,8 +211,6 @@ class TestNetworkQualityEstimator : public NetworkQualityEstimator {
     transport_rtt_observation_count_last_ect_computation_ = count;
   }
 
-  void SetCurrentSignalStrength(int32_t signal_strength);
-
   // Returns count of ping RTTs received from H2/spdy connections.
   size_t ping_rtt_received_count() const { return ping_rtt_received_count_; }
 
@@ -228,10 +226,6 @@ class TestNetworkQualityEstimator : public NetworkQualityEstimator {
   using NetworkQualityEstimator::AddAndNotifyObserversOfThroughput;
   using NetworkQualityEstimator::IsHangingRequest;
 
-  // NetworkQualityEstimator implementation that returns the overridden
-  // network id and signal strength (instead of invoking platform APIs).
-  absl::optional<int32_t> GetCurrentSignalStrengthWithThrottling() override;
-
  private:
   class LocalHttpTestServer : public EmbeddedTestServer {
    public:
@@ -284,8 +278,6 @@ class TestNetworkQualityEstimator : public NetworkQualityEstimator {
   // If set, GetRTTEstimateInternal() would return the set value.
   absl::optional<base::TimeDelta> start_time_null_end_to_end_rtt_;
 
-  absl::optional<int32_t> current_cellular_signal_strength_;
-
   LocalHttpTestServer embedded_test_server_;
 
   // If true, notifications are not sent to any of the observers.
