@@ -49,9 +49,6 @@
 
 namespace base {
 class Value;
-namespace trace_event {
-class ProcessMemoryDump;
-}
 }  // namespace base
 
 namespace quic {
@@ -373,11 +370,6 @@ class NET_EXPORT_PRIVATE QuicStreamFactory
     return default_network_;
   }
 
-  // Dumps memory allocation stats. |parent_dump_absolute_name| is the name
-  // used by the parent MemoryAllocatorDump in the memory dump hierarchy.
-  void DumpMemoryStats(base::trace_event::ProcessMemoryDump* pmd,
-                       const std::string& parent_absolute_name) const;
-
   // Returns the stored DNS aliases for the session key.
   const std::vector<std::string>& GetDnsAliasesForSessionKey(
       const QuicSessionKey& key) const;
