@@ -10,6 +10,7 @@
 #include <utility>
 
 #include "base/containers/contains.h"
+#include "base/ranges/algorithm.h"
 #include "base/stl_util.h"
 #include "base/time/clock.h"
 #include "base/time/tick_clock.h"
@@ -27,6 +28,7 @@ ReportingCacheImpl::ReportingCacheImpl(ReportingContext* context)
 ReportingCacheImpl::~ReportingCacheImpl() = default;
 
 void ReportingCacheImpl::AddReport(
+    const absl::optional<base::UnguessableToken>& reporting_source,
     const NetworkIsolationKey& network_isolation_key,
     const GURL& url,
     const std::string& user_agent,
@@ -36,9 +38,11 @@ void ReportingCacheImpl::AddReport(
     int depth,
     base::TimeTicks queued,
     int attempts) {
+  // If |reporting_source| is present, it must not be empty.
+  DCHECK(!(reporting_source.has_value() && reporting_source->is_empty()));
   auto report = std::make_unique<ReportingReport>(
-      absl::nullopt, network_isolation_key, url, user_agent, group_name, type,
-      std::move(body), depth, queued, attempts);
+      reporting_source, network_isolation_key, url, user_agent, group_name,
+      type, std::move(body), depth, queued, attempts);
 
   auto inserted = reports_.insert(std::move(report));
   DCHECK(inserted.second);
@@ -51,7 +55,12 @@ void ReportingCacheImpl::AddReport(
     // The newly-added report isn't pending, so even if all other reports are
     // pending, the cache should have a report to evict.
     DCHECK(!to_evict->get()->IsUploadPending());
+    if (to_evict != inserted.first) {
+      context_->NotifyReportAdded(inserted.first->get());
+    }
     reports_.erase(to_evict);
+  } else {
+    context_->NotifyReportAdded(inserted.first->get());
   }
 
   context_->NotifyCachedReportsUpdated();
@@ -61,7 +70,8 @@ void ReportingCacheImpl::GetReports(
     std::vector<const ReportingReport*>* reports_out) const {
   reports_out->clear();
   for (const auto& report : reports_) {
-    if (report->status != ReportingReport::Status::DOOMED)
+    if (report->status != ReportingReport::Status::DOOMED &&
+        report->status != ReportingReport::Status::SUCCESS)
       reports_out->push_back(report.get());
   }
 }
@@ -105,6 +115,9 @@ base::Value ReportingCacheImpl::GetReportsAsValue() const {
       case ReportingReport::Status::QUEUED:
         report_dict.SetKey("status", base::Value("queued"));
         break;
+      case ReportingReport::Status::SUCCESS:
+        report_dict.SetKey("status", base::Value("success"));
+        break;
     }
     report_list.push_back(std::move(report_dict));
   }
@@ -117,11 +130,29 @@ std::vector<const ReportingReport*> ReportingCacheImpl::GetReportsToDeliver() {
     if (report->IsUploadPending())
       continue;
     report->status = ReportingReport::Status::PENDING;
+    context_->NotifyReportUpdated(report.get());
     reports_out.push_back(report.get());
   }
   return reports_out;
 }
 
+std::vector<const ReportingReport*>
+ReportingCacheImpl::GetReportsToDeliverForSource(
+    const base::UnguessableToken& reporting_source) {
+  DCHECK(!reporting_source.is_empty());
+  std::vector<const ReportingReport*> reports_out;
+  for (const auto& report : reports_) {
+    if (report->reporting_source == reporting_source) {
+      if (report->IsUploadPending())
+        continue;
+      report->status = ReportingReport::Status::PENDING;
+      context_->NotifyReportUpdated(report.get());
+      reports_out.push_back(report.get());
+    }
+  }
+  return reports_out;
+}
+
 void ReportingCacheImpl::ClearReportsPending(
     const std::vector<const ReportingReport*>& reports) {
   std::vector<const ReportingReport*> reports_to_remove;
@@ -129,11 +160,13 @@ void ReportingCacheImpl::ClearReportsPending(
   for (const ReportingReport* report : reports) {
     auto it = reports_.find(report);
     DCHECK(it != reports_.end());
-    if (it->get()->status == ReportingReport::Status::DOOMED) {
+    if (it->get()->status == ReportingReport::Status::DOOMED ||
+        it->get()->status == ReportingReport::Status::SUCCESS) {
       reports_.erase(it);
     } else {
       DCHECK_EQ(ReportingReport::Status::PENDING, it->get()->status);
       it->get()->status = ReportingReport::Status::QUEUED;
+      context_->NotifyReportUpdated(it->get());
     }
   }
 }
@@ -144,6 +177,7 @@ void ReportingCacheImpl::IncrementReportsAttempts(
     auto it = reports_.find(report);
     DCHECK(it != reports_.end());
     it->get()->attempts++;
+    context_->NotifyReportUpdated(it->get());
   }
 
   context_->NotifyCachedReportsUpdated();
@@ -169,16 +203,49 @@ void ReportingCacheImpl::IncrementEndpointDeliveries(
   }
 }
 
+void ReportingCacheImpl::SetExpiredSource(
+    const base::UnguessableToken& reporting_source) {
+  DCHECK(!reporting_source.is_empty());
+  expired_sources_.insert(reporting_source);
+}
+
+const base::flat_set<base::UnguessableToken>&
+ReportingCacheImpl::GetExpiredSources() const {
+  return expired_sources_;
+}
+
 void ReportingCacheImpl::RemoveReports(
     const std::vector<const ReportingReport*>& reports) {
+  RemoveReports(reports, false);
+}
+
+void ReportingCacheImpl::RemoveReports(
+    const std::vector<const ReportingReport*>& reports,
+    bool delivery_success) {
   for (const ReportingReport* report : reports) {
     auto it = reports_.find(report);
     DCHECK(it != reports_.end());
 
-    if (it->get()->IsUploadPending()) {
-      it->get()->status = ReportingReport::Status::DOOMED;
-    } else {
-      reports_.erase(it);
+    switch (it->get()->status) {
+      case ReportingReport::Status::DOOMED:
+        if (delivery_success) {
+          it->get()->status = ReportingReport::Status::SUCCESS;
+          context_->NotifyReportUpdated(it->get());
+        }
+        break;
+      case ReportingReport::Status::PENDING:
+        it->get()->status = delivery_success ? ReportingReport::Status::SUCCESS
+                                             : ReportingReport::Status::DOOMED;
+        context_->NotifyReportUpdated(it->get());
+        break;
+      case ReportingReport::Status::QUEUED:
+        it->get()->status = delivery_success ? ReportingReport::Status::SUCCESS
+                                             : ReportingReport::Status::DOOMED;
+        context_->NotifyReportUpdated(it->get());
+        reports_.erase(it);
+        break;
+      case ReportingReport::Status::SUCCESS:
+        break;
     }
   }
   context_->NotifyCachedReportsUpdated();
@@ -194,6 +261,16 @@ size_t ReportingCacheImpl::GetFullReportCountForTesting() const {
   return reports_.size();
 }
 
+size_t ReportingCacheImpl::GetReportCountWithStatusForTesting(
+    ReportingReport::Status status) const {
+  size_t count = 0;
+  for (const auto& report : reports_) {
+    if (report->status == status)
+      ++count;
+  }
+  return count;
+}
+
 bool ReportingCacheImpl::IsReportPendingForTesting(
     const ReportingReport* report) const {
   DCHECK(report);
@@ -205,7 +282,8 @@ bool ReportingCacheImpl::IsReportDoomedForTesting(
     const ReportingReport* report) const {
   DCHECK(report);
   DCHECK(reports_.find(report) != reports_.end());
-  return report->status == ReportingReport::Status::DOOMED;
+  return report->status == ReportingReport::Status::DOOMED ||
+         report->status == ReportingReport::Status::SUCCESS;
 }
 
 void ReportingCacheImpl::OnParsedHeader(
@@ -272,6 +350,33 @@ void ReportingCacheImpl::OnParsedHeader(
   context_->NotifyCachedClientsUpdated();
 }
 
+void ReportingCacheImpl::RemoveSourceAndEndpoints(
+    const base::UnguessableToken& reporting_source) {
+  DCHECK(!reporting_source.is_empty());
+  // Sanity checks: The source must be in the list of expired sources, and
+  // there must be no more cached reports for it (except reports already marked
+  // as doomed, as they will be garbage collected soon).
+  DCHECK(expired_sources_.contains(reporting_source));
+  DCHECK(
+      base::ranges::none_of(reports_, [reporting_source](const auto& report) {
+        return report->reporting_source == reporting_source &&
+               report->status != ReportingReport::Status::DOOMED &&
+               report->status != ReportingReport::Status::SUCCESS;
+      }));
+  document_endpoints_.erase(reporting_source);
+  expired_sources_.erase(reporting_source);
+  context_->NotifyEndpointsUpdated();
+}
+
+void ReportingCacheImpl::OnParsedReportingEndpointsHeader(
+    const base::UnguessableToken& reporting_source,
+    std::vector<ReportingEndpoint> endpoints) {
+  DCHECK(!reporting_source.is_empty());
+  DCHECK(!endpoints.empty());
+  document_endpoints_.insert({reporting_source, std::move(endpoints)});
+  context_->NotifyEndpointsUpdated();
+}
+
 std::set<url::Origin> ReportingCacheImpl::GetAllOrigins() const {
   ConsistencyCheckClients();
   std::set<url::Origin> origins_out;
@@ -472,16 +577,54 @@ void ReportingCacheImpl::AddClientsLoadedFromStore(
   ConsistencyCheckClients();
 }
 
+// Until the V0 Reporting API is deprecated and removed, this method needs to
+// handle endpoint groups configured by both the V0 Report-To header, which are
+// persisted and used by any resource on the origin which defined them, as well
+// as the V1 Reporting-Endpoints header, which defines ephemeral endpoints
+// which can only be used by the resource which defines them.
+// In order to properly isolate reports from different documents, any reports
+// which can be sent to a V1 endpoint must be. V0 endpoints are selected only
+// for those reports with no reporting source token, or when no matching V1
+// endpoint has been configured.
+// To achieve this, the reporting service continues to use the EndpointGroupKey
+// structure, which uses the presence of an optional reporting source token to
+// distinguish V1 endpoints from V0 endpoint groups.
 std::vector<ReportingEndpoint>
 ReportingCacheImpl::GetCandidateEndpointsForDelivery(
     const ReportingEndpointGroupKey& group_key) {
   base::Time now = clock().Now();
   ConsistencyCheckClients();
 
+  // If |group_key| has a defined |reporting_source| field, then this method is
+  // being called for reports with an associated source. We need to first look
+  // for a matching V1 endpoint, based on |reporting_source| and |group_name|.
+  if (group_key.IsDocumentEndpoint()) {
+    const auto it =
+        document_endpoints_.find(group_key.reporting_source.value());
+    if (it != document_endpoints_.end()) {
+      for (const ReportingEndpoint& endpoint : it->second) {
+        if (endpoint.group_key == group_key) {
+          return {endpoint};
+        }
+      }
+    }
+  }
+
+  // Either |group_key| does not have a defined |reporting_source|, which means
+  // that this method was called for reports without a source (e.g. NEL), or
+  // we tried and failed to find an appropriate V1 endpoint. In either case, we
+  // now look for the appropriate V0 endpoints.
+
+  // We need to clear out the |reporting_source| field to get a group key which
+  // can be compared to any V0 endpoint groups.
+  ReportingEndpointGroupKey v0_lookup_group_key(
+      group_key.network_isolation_key, group_key.origin, group_key.group_name);
+
   // Look for an exact origin match for |origin| and |group|.
-  EndpointGroupMap::iterator group_it = FindEndpointGroupIt(group_key);
+  EndpointGroupMap::iterator group_it =
+      FindEndpointGroupIt(v0_lookup_group_key);
   if (group_it != endpoint_groups_.end() && group_it->second.expires > now) {
-    ClientMap::iterator client_it = FindClientIt(group_key);
+    ClientMap::iterator client_it = FindClientIt(v0_lookup_group_key);
     MarkEndpointGroupAndClientUsed(client_it, group_it, now);
     ConsistencyCheckClients();
     context_->NotifyCachedClientsUpdated();
@@ -491,18 +634,20 @@ ReportingCacheImpl::GetCandidateEndpointsForDelivery(
   // If no endpoints were found for an exact match, look for superdomain matches
   // TODO(chlily): Limit the number of labels to go through when looking for a
   // superdomain match.
-  std::string domain = group_key.origin.host();
+  std::string domain = v0_lookup_group_key.origin.host();
   while (!domain.empty()) {
     const auto domain_range = clients_.equal_range(domain);
     for (auto client_it = domain_range.first; client_it != domain_range.second;
          ++client_it) {
       // Client for a superdomain of |origin|
       const Client& client = client_it->second;
-      if (client.network_isolation_key != group_key.network_isolation_key)
+      if (client.network_isolation_key !=
+          v0_lookup_group_key.network_isolation_key)
         continue;
-      ReportingEndpointGroupKey new_group(group_key.network_isolation_key,
-                                          client.origin, group_key.group_name);
-      group_it = FindEndpointGroupIt(new_group);
+      ReportingEndpointGroupKey superdomain_lookup_group_key(
+          v0_lookup_group_key.network_isolation_key, client.origin,
+          v0_lookup_group_key.group_name);
+      group_it = FindEndpointGroupIt(superdomain_lookup_group_key);
 
       if (group_it == endpoint_groups_.end())
         continue;
@@ -514,7 +659,7 @@ ReportingCacheImpl::GetCandidateEndpointsForDelivery(
         MarkEndpointGroupAndClientUsed(client_it, group_it, now);
         ConsistencyCheckClients();
         context_->NotifyCachedClientsUpdated();
-        return GetEndpointsInGroup(new_group);
+        return GetEndpointsInGroup(superdomain_lookup_group_key);
       }
     }
     domain = GetSuperdomain(domain);
@@ -541,6 +686,20 @@ void ReportingCacheImpl::Flush() {
     store()->Flush();
 }
 
+ReportingEndpoint ReportingCacheImpl::GetV1EndpointForTesting(
+    const base::UnguessableToken& reporting_source,
+    const std::string& endpoint_name) const {
+  DCHECK(!reporting_source.is_empty());
+  const auto it = document_endpoints_.find(reporting_source);
+  if (it != document_endpoints_.end()) {
+    for (const ReportingEndpoint& endpoint : it->second) {
+      if (endpoint_name == endpoint.group_key.group_name)
+        return endpoint;
+    }
+  }
+  return ReportingEndpoint();
+}
+
 ReportingEndpoint ReportingCacheImpl::GetEndpointForTesting(
     const ReportingEndpointGroupKey& group_key,
     const GURL& url) const {
@@ -593,6 +752,33 @@ size_t ReportingCacheImpl::GetClientCountForTesting() const {
   return clients_.size();
 }
 
+size_t ReportingCacheImpl::GetReportingSourceCountForTesting() const {
+  return document_endpoints_.size();
+}
+
+void ReportingCacheImpl::SetV1EndpointForTesting(
+    const ReportingEndpointGroupKey& group_key,
+    const base::UnguessableToken& reporting_source,
+    const GURL& url) {
+  DCHECK(!reporting_source.is_empty());
+  DCHECK(group_key.IsDocumentEndpoint());
+  DCHECK_EQ(reporting_source, group_key.reporting_source.value());
+  ReportingEndpoint::EndpointInfo info;
+  info.url = url;
+  ReportingEndpoint new_endpoint(group_key, info);
+  if (document_endpoints_.count(reporting_source) > 0) {
+    // The endpoints list is const, so remove and replace with an updated list.
+    std::vector<ReportingEndpoint> endpoints =
+        document_endpoints_.at(reporting_source);
+    endpoints.push_back(std::move(new_endpoint));
+    document_endpoints_.erase(reporting_source);
+    document_endpoints_.insert({reporting_source, std::move(endpoints)});
+  } else {
+    document_endpoints_.insert({reporting_source, {std::move(new_endpoint)}});
+  }
+  context_->NotifyEndpointsUpdated();
+}
+
 void ReportingCacheImpl::SetEndpointForTesting(
     const ReportingEndpointGroupKey& group_key,
     const GURL& url,
@@ -737,6 +923,9 @@ size_t ReportingCacheImpl::ConsistencyCheckClient(const std::string& domain,
     size_t groups_with_name = 0;
     for (const auto& key_and_group : endpoint_groups_) {
       const ReportingEndpointGroupKey& key = key_and_group.first;
+      // There should not be any V1 document endpoints; this is a V0 endpoint
+      // group.
+      DCHECK(!key_and_group.first.IsDocumentEndpoint());
       if (key.origin == client.origin &&
           key.network_isolation_key == client.network_isolation_key &&
           key.group_name == group_name) {
@@ -914,7 +1103,7 @@ void ReportingCacheImpl::AddOrUpdateEndpoint(ReportingEndpoint new_endpoint) {
     if (context_->IsClientDataPersisted())
       store()->AddReportingEndpoint(new_endpoint);
 
-    EndpointMap::iterator endpoint_it = endpoints_.insert(
+    endpoint_it = endpoints_.insert(
         std::make_pair(new_endpoint.group_key, std::move(new_endpoint)));
     AddEndpointItToIndex(endpoint_it);
 
