@@ -36,12 +36,6 @@
 #include "net/third_party/quiche/src/spdy/core/spdy_protocol.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 
-namespace base {
-namespace trace_event {
-class ProcessMemoryDump;
-}
-}
-
 namespace net {
 
 class ClientSocketHandle;
@@ -304,9 +298,6 @@ class NET_EXPORT SpdySessionPool
   // streams.
   void OnSSLConfigForServerChanged(const HostPortPair& server) override;
 
-  void DumpMemoryStats(base::trace_event::ProcessMemoryDump* pmd,
-                       const std::string& parent_dump_absolute_name) const;
-
   void set_network_quality_estimator(
       NetworkQualityEstimator* network_quality_estimator) {
     network_quality_estimator_ = network_quality_estimator;
