@@ -190,8 +190,6 @@ const base::Value& BaseTestServer::server_data() const {
 
 std::string BaseTestServer::GetScheme() const {
   switch (type_) {
-    case TYPE_FTP:
-      return "ftp";
     case TYPE_HTTP:
       return "http";
     case TYPE_HTTPS:
@@ -429,11 +427,6 @@ bool BaseTestServer::GenerateArguments(base::DictionaryValue* arguments) const {
     arguments->SetKey("ws-basic-auth", base::Value());
   }
 
-  if (no_anonymous_ftp_user_) {
-    DCHECK_EQ(TYPE_FTP, type_);
-    arguments->SetKey("no-anonymous-ftp-user", base::Value());
-  }
-
   if (redirect_connect_to_localhost_) {
     DCHECK(type_ == TYPE_BASIC_AUTH_PROXY || type_ == TYPE_PROXY);
     arguments->SetKey("redirect-connect-to-localhost", base::Value());
