@@ -11,6 +11,7 @@
 #include "base/bind.h"
 #include "base/callback_helpers.h"
 #include "base/check_op.h"
+#include "base/containers/contains.h"
 #include "base/notreached.h"
 #include "base/sequenced_task_runner.h"
 #include "base/single_thread_task_runner.h"
@@ -24,7 +25,9 @@
 #include "net/base/upload_bytes_element_reader.h"
 #include "net/base/upload_data_stream.h"
 #include "net/base/upload_file_element_reader.h"
+#include "net/cert/x509_certificate.h"
 #include "net/http/http_response_headers.h"
+#include "net/ssl/ssl_private_key.h"
 #include "net/url_request/redirect_info.h"
 #include "net/url_request/url_fetcher_delegate.h"
 #include "net/url_request/url_fetcher_response_writer.h"
@@ -81,7 +84,7 @@ URLFetcherCore::URLFetcherCore(
       delegate_(d),
       delegate_task_runner_(base::SequencedTaskRunnerHandle::Get()),
       load_flags_(LOAD_NORMAL),
-      allow_credentials_(base::nullopt),
+      allow_credentials_(absl::nullopt),
       response_code_(URLFetcher::RESPONSE_CODE_INVALID),
       url_request_data_key_(nullptr),
       was_cached_(false),
@@ -217,7 +220,7 @@ void URLFetcherCore::SetLoadFlags(int load_flags) {
 }
 
 void URLFetcherCore::SetAllowCredentials(bool allow_credentials) {
-  allow_credentials_ = base::make_optional<bool>(allow_credentials);
+  allow_credentials_ = absl::make_optional<bool>(allow_credentials);
 }
 
 int URLFetcherCore::GetLoadFlags() const {
@@ -249,7 +252,7 @@ void URLFetcherCore::SetRequestContext(
 }
 
 void URLFetcherCore::SetInitiator(
-    const base::Optional<url::Origin>& initiator) {
+    const absl::optional<url::Origin>& initiator) {
   DCHECK(!initiator_.has_value());
   initiator_ = initiator;
 }
