@@ -236,11 +236,8 @@ GURL GURL::ReplaceComponents(
       NULL, &output, &result.parsed_);
 
   output.Complete();
-  if (result.is_valid_ && result.SchemeIsFileSystem()) {
-    result.inner_url_ =
-        std::make_unique<GURL>(result.spec_.data(), result.parsed_.Length(),
-                               *result.parsed_.inner_parsed(), true);
-  }
+
+  ProcessFileOrFileSystemURLAfterReplaceComponents(result);
   return result;
 }
 
@@ -259,14 +256,34 @@ GURL GURL::ReplaceComponents(
       NULL, &output, &result.parsed_);
 
   output.Complete();
-  if (result.is_valid_ && result.SchemeIsFileSystem()) {
-    result.inner_url_ =
-        std::make_unique<GURL>(result.spec_.data(), result.parsed_.Length(),
-                               *result.parsed_.inner_parsed(), true);
-  }
+
+  ProcessFileOrFileSystemURLAfterReplaceComponents(result);
+
   return result;
 }
 
+void GURL::ProcessFileOrFileSystemURLAfterReplaceComponents(GURL& url) const {
+  if (!url.is_valid_)
+    return;
+  if (url.SchemeIsFileSystem()) {
+    url.inner_url_ =
+        std::make_unique<GURL>(url.spec_.data(), url.parsed_.Length(),
+                               *url.parsed_.inner_parsed(), true);
+  }
+#ifdef WIN32
+  if (url.SchemeIsFile()) {
+    // On Win32, some file URLs created through ReplaceComponents used to lose
+    // its hostname after getting reparsed (e.g. when it's sent through IPC) due
+    // to special handling of file URLs with Windows-drive paths in the URL
+    // parser. To make the behavior for URLs modified through ReplaceComponents
+    // (instead of getting fully reparsed) the same, immediately reparse the
+    // URL here to trigger the special handling.
+    // See https://crbug.com/1214098.
+    url = GURL(url.spec());
+  }
+#endif
+}
+
 GURL GURL::GetOrigin() const {
   // This doesn't make sense for invalid or nonstandard URLs, so return
   // the empty URL.
